@@ -37,7 +37,7 @@ impl Year {
     /// assert_eq!(Year(2000).is_leap_year(), true);
     /// assert_eq!(Year(1900).is_leap_year(), false);
     /// ```
-    pub fn is_leap_year(&self) -> bool {
+    pub fn is_leap_year(self) -> bool {
         self.leap_year_calculations().1
     }
 
@@ -69,9 +69,9 @@ impl Year {
     /// assert_eq!(year.months(April .. June).count(), 2);
     /// assert_eq!(year.months(.. June).count(), 5);
     /// ```
-    pub fn months<S: MonthSpan>(&self, span: S) -> YearMonths {
+    pub fn months<S: MonthSpan>(self, span: S) -> YearMonths {
         YearMonths {
-            year: *self,
+            year: self,
             iter: span.get_slice().iter(),
         }
     }
@@ -87,10 +87,10 @@ impl Year {
     /// assert_eq!(*expiry_date.year, 2017);
     /// assert_eq!(expiry_date.month, Month::February);
     /// ```
-    pub fn month(&self, month: Month) -> YearMonth {
+    pub fn month(self, month: Month) -> YearMonth {
         YearMonth {
-            year: *self,
-            month: month,
+            year: self,
+            month,
         }
     }
 
@@ -99,7 +99,7 @@ impl Year {
     ///
     /// 1. The number of leap years that have elapsed prior to this year;
     /// 2. Whether this year is a leap year or not.
-    fn leap_year_calculations(&self) -> (i64, bool) {
+    fn leap_year_calculations(self) -> (i64, bool) {
         let year = self.0 - 2000;
 
         // This calculation is the reverse of LocalDate::from_days_since_epoch.
@@ -123,7 +123,7 @@ impl Year {
 impl Deref for Year {
     type Target = i64;
 
-    fn deref<'a>(&'a self) -> &'a Self::Target {
+    fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
@@ -137,7 +137,7 @@ pub trait MonthSpan {
     fn get_slice(&self) -> &'static [Month];
 }
 
-static MONTHS: &'static [Month] = &[
+static MONTHS: &[Month] = &[
     January,  February,  March,
     April,    May,       June,
     July,     August,    September,
@@ -355,15 +355,15 @@ const SECONDS_IN_DAY: i64 = 86400;
 ///
 /// [^win32]: http://blogs.msdn.com/b/oldnewthing/archive/2009/03/06/9461176.aspx
 ///
-const EPOCH_DIFFERENCE: i64 = (30 * 365      // 30 years between 2000 and 1970...
-                               + 7           // plus seven days for leap years...
-                               + 31 + 29);   // plus all the days in January and February in 2000.
+const EPOCH_DIFFERENCE: i64 = 30 * 365   // 30 years between 2000 and 1970...
+                            + 7          // plus seven days for leap years...
+                            + 31 + 29;   // plus all the days in January and February in 2000.
 
 
 /// This rather strange triangle is an array of the number of days elapsed
 /// at the end of each month, starting at the beginning of March (the first
 /// month after the EPOCH above), going backwards, ignoring February.
-const TIME_TRIANGLE: &'static [i64; 11] =
+const TIME_TRIANGLE: &[i64; 11] =
     &[31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 + 31 + 31,  // January
       31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30 + 31,  // December
       31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30,  // November
@@ -429,10 +429,10 @@ impl LocalDate {
     ///
     /// assert!(LocalDate::ymd(2100, Month::February, 29).is_err());
     /// ```
-    pub fn ymd(year: i64, month: Month, day: i8) -> Result<LocalDate, Error> {
-        YMD { year: year, month: month, day: day }
+    pub fn ymd(year: i64, month: Month, day: i8) -> Result<Self, Error> {
+        YMD { year, month, day }
             .to_days_since_epoch()
-            .map(|days| LocalDate::from_days_since_epoch(days - EPOCH_DIFFERENCE))
+            .map(|days| Self::from_days_since_epoch(days - EPOCH_DIFFERENCE))
     }
 
     /// Creates a new local date instance from the given year and day-of-year
@@ -454,11 +454,11 @@ impl LocalDate {
     /// assert_eq!(date.month(), Month::September);
     /// assert_eq!(date.day(), 13);
     /// ```
-    pub fn yd(year: i64, yearday: i64) -> Result<LocalDate, Error> {
+    pub fn yd(year: i64, yearday: i64) -> Result<Self, Error> {
         if yearday.is_within(0..367) {
-            let jan_1 = YMD { year: year, month: January, day: 1 };
-            let days = try!(jan_1.to_days_since_epoch());
-            Ok(LocalDate::from_days_since_epoch(days + yearday - 1 - EPOCH_DIFFERENCE))
+            let jan_1 = YMD { year, month: January, day: 1 };
+            let days = jan_1.to_days_since_epoch()?;
+            Ok(Self::from_days_since_epoch(days + yearday - 1 - EPOCH_DIFFERENCE))
         }
         else {
             Err(Error::OutOfRange)
@@ -504,24 +504,24 @@ impl LocalDate {
     /// assert_eq!(date.day(), 3);
     /// assert_eq!(date.weekday(), Weekday::Sunday);
     /// ```
-    pub fn ywd(year: i64, week: i64, weekday: Weekday) -> Result<LocalDate, Error> {
-        let jan_4 = YMD { year: year, month: January, day: 4 };
+    pub fn ywd(year: i64, week: i64, weekday: Weekday) -> Result<Self, Error> {
+        let jan_4 = YMD { year, month: January, day: 4 };
         let correction = days_to_weekday(jan_4.to_days_since_epoch().unwrap() - EPOCH_DIFFERENCE).days_from_monday_as_one() as i64 + 3;
 
         let yearday = 7 * week + weekday.days_from_monday_as_one() as i64 - correction;
 
         if yearday <= 0 {
             let days_in_year = if Year(year - 1).is_leap_year() { 366 } else { 365 };
-            LocalDate::yd(year - 1, days_in_year + yearday)
+            Self::yd(year - 1, days_in_year + yearday)
         }
         else {
             let days_in_year = if Year(year).is_leap_year() { 366 } else { 365 };
 
             if yearday >= days_in_year {
-                LocalDate::yd(year + 1, yearday - days_in_year)
+                Self::yd(year + 1, yearday - days_in_year)
             }
             else {
-                LocalDate::yd(year, yearday)
+                Self::yd(year, yearday)
             }
         }
     }
@@ -553,7 +553,7 @@ impl LocalDate {
     /// assert_eq!(date.month(), Month::September);
     /// assert_eq!(date.day(), 24);  // not the 25th!
     /// ```
-    fn from_days_since_epoch(days: i64) -> LocalDate {
+    fn from_days_since_epoch(days: i64) -> Self {
 
         // The Gregorian calendar works in 400-year cycles, which repeat
         // themselves ever after.
@@ -572,7 +572,7 @@ impl LocalDate {
         let num_4y_cycles = remainder / DAYS_IN_4Y;
         remainder -= num_4y_cycles * DAYS_IN_4Y;  // remainder is now days left in this 4-year cycle
 
-        let mut years = remainder / 365;
+        let mut years = std::cmp::min(remainder / 365, 3);
         remainder -= years * 365;  // remainder is now days left in this year
 
         // Leap year calculation goes thusly:
@@ -629,7 +629,7 @@ impl LocalDate {
         // Finally, adjust the day numbers for human reasons: the first day
         // of the month is the 1st, rather than the 0th, and the year needs
         // to be adjusted relative to the EPOCH.
-        LocalDate {
+        Self {
             yearday: (day_of_year + 1) as i16,
             weekday: days_to_weekday(days),
             ymd: YMD {
@@ -651,11 +651,11 @@ impl LocalDate {
     ///
     /// For this reason, the function is marked as `unsafe`, even though it
     /// (technically) uses unsafe components.
-    pub unsafe fn _new_with_prefilled_values(year: i64, month: Month, day: i8, weekday: Weekday, yearday: i16) -> LocalDate {
-        LocalDate {
-            ymd: YMD { year: year, month: month, day: day },
-            weekday: weekday,
-            yearday: yearday,
+    pub unsafe fn _new_with_prefilled_values(year: i64, month: Month, day: i8, weekday: Weekday, yearday: i16) -> Self {
+        Self {
+            ymd: YMD { year, month, day },
+            weekday,
+            yearday,
         }
     }
 
@@ -678,19 +678,19 @@ impl fmt::Debug for LocalDate {
 }
 
 impl PartialEq for LocalDate {
-    fn eq(&self, other: &LocalDate) -> bool {
+    fn eq(&self, other: &Self) -> bool {
         self.ymd == other.ymd
     }
 }
 
 impl PartialOrd for LocalDate {
-    fn partial_cmp(&self, other: &LocalDate) -> Option<Ordering> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         self.ymd.partial_cmp(&other.ymd)
     }
 }
 
 impl Ord for LocalDate {
-    fn cmp(&self, other: &LocalDate) -> Ordering {
+    fn cmp(&self, other: &Self) -> Ordering {
         self.ymd.cmp(&other.ymd)
     }
 }
@@ -699,14 +699,14 @@ impl LocalTime {
 
     /// Computes the number of hours, minutes, and seconds, based on the
     /// number of seconds that have elapsed since midnight.
-    pub fn from_seconds_since_midnight(seconds: i64) -> LocalTime {
-        LocalTime::from_seconds_and_milliseconds_since_midnight(seconds, 0)
+    pub fn from_seconds_since_midnight(seconds: i64) -> Self {
+        Self::from_seconds_and_milliseconds_since_midnight(seconds, 0)
     }
 
     /// Computes the number of hours, minutes, and seconds, based on the
     /// number of seconds that have elapsed since midnight.
-    pub fn from_seconds_and_milliseconds_since_midnight(seconds: i64, millisecond_of_second: i16) -> LocalTime {
-        LocalTime {
+    pub fn from_seconds_and_milliseconds_since_midnight(seconds: i64, millisecond_of_second: i16) -> Self {
+        Self {
             hour:   (seconds / 60 / 60) as i8,
             minute: (seconds / 60 % 60) as i8,
             second: (seconds % 60) as i8,
@@ -715,8 +715,8 @@ impl LocalTime {
     }
 
     /// Returns the time at midnight, with all fields initialised to 0.
-    pub fn midnight() -> LocalTime {
-        LocalTime { hour: 0, minute: 0, second: 0, millisecond: 0 }
+    pub fn midnight() -> Self {
+        Self { hour: 0, minute: 0, second: 0, millisecond: 0 }
     }
 
     /// Creates a new timestamp instance with the given hour and minute
@@ -724,10 +724,10 @@ impl LocalTime {
     ///
     /// The values are checked for validity before instantiation, and
     /// passing in values out of range will return an `Err`.
-    pub fn hm(hour: i8, minute: i8) -> Result<LocalTime, Error> {
+    pub fn hm(hour: i8, minute: i8) -> Result<Self, Error> {
         if (hour.is_within(0..24) && minute.is_within(0..60))
         || (hour == 24 && minute == 00) {
-            Ok(LocalTime { hour: hour, minute: minute, second: 0, millisecond: 0 })
+            Ok(Self { hour, minute, second: 0, millisecond: 0 })
         }
         else {
             Err(Error::OutOfRange)
@@ -739,10 +739,10 @@ impl LocalTime {
     ///
     /// The values are checked for validity before instantiation, and
     /// passing in values out of range will return an `Err`.
-    pub fn hms(hour: i8, minute: i8, second: i8) -> Result<LocalTime, Error> {
+    pub fn hms(hour: i8, minute: i8, second: i8) -> Result<Self, Error> {
         if (hour.is_within(0..24) && minute.is_within(0..60) && second.is_within(0..60))
         || (hour == 24 && minute == 00 && second == 00) {
-            Ok(LocalTime { hour: hour, minute: minute, second: second, millisecond: 0 })
+            Ok(Self { hour, minute, second, millisecond: 0 })
         }
         else {
             Err(Error::OutOfRange)
@@ -754,11 +754,11 @@ impl LocalTime {
     ///
     /// The values are checked for validity before instantiation, and
     /// passing in values out of range will return an `Err`.
-    pub fn hms_ms(hour: i8, minute: i8, second: i8, millisecond: i16) -> Result<LocalTime, Error> {
+    pub fn hms_ms(hour: i8, minute: i8, second: i8, millisecond: i16) -> Result<Self, Error> {
         if hour.is_within(0..24)   && minute.is_within(0..60)
         && second.is_within(0..60) && millisecond.is_within(0..1000)
         {
-            Ok(LocalTime { hour: hour, minute: minute, second: second, millisecond: millisecond })
+            Ok(Self { hour, minute, second, millisecond })
         }
         else {
             Err(Error::OutOfRange)
@@ -767,7 +767,7 @@ impl LocalTime {
 
     /// Calculate the number of seconds since midnight this time is at,
     /// ignoring milliseconds.
-    pub fn to_seconds(&self) -> i64 {
+    pub fn to_seconds(self) -> i64 {
         self.hour as i64 * 3600
             + self.minute as i64 * 60
             + self.second as i64
@@ -792,37 +792,37 @@ impl LocalDateTime {
 
     /// Computes a complete date-time based on the values in the given
     /// Instant parameter.
-    pub fn from_instant(instant: Instant) -> LocalDateTime {
-        LocalDateTime::at_ms(instant.seconds(), instant.milliseconds())
+    pub fn from_instant(instant: Instant) -> Self {
+        Self::at_ms(instant.seconds(), instant.milliseconds())
     }
 
     /// Computes a complete date-time based on the number of seconds that
     /// have elapsed since **midnight, 1st January, 1970**, setting the
     /// number of milliseconds to 0.
-    pub fn at(seconds_since_1970_epoch: i64) -> LocalDateTime {
-        LocalDateTime::at_ms(seconds_since_1970_epoch, 0)
+    pub fn at(seconds_since_1970_epoch: i64) -> Self {
+        Self::at_ms(seconds_since_1970_epoch, 0)
     }
 
     /// Computes a complete date-time based on the number of seconds that
     /// have elapsed since **midnight, 1st January, 1970**,
-    pub fn at_ms(seconds_since_1970_epoch: i64, millisecond_of_second: i16) -> LocalDateTime {
+    pub fn at_ms(seconds_since_1970_epoch: i64, millisecond_of_second: i16) -> Self {
         let seconds = seconds_since_1970_epoch - EPOCH_DIFFERENCE * SECONDS_IN_DAY;
 
         // Just split the input value into days and seconds, and let
         // LocalDate and LocalTime do all the hard work.
         let (days, secs) = split_cycles(seconds, SECONDS_IN_DAY);
 
-        LocalDateTime {
+        Self {
             date: LocalDate::from_days_since_epoch(days),
             time: LocalTime::from_seconds_and_milliseconds_since_midnight(secs, millisecond_of_second),
         }
     }
 
     /// Creates a new local date time from a local date and a local time.
-    pub fn new(date: LocalDate, time: LocalTime) -> LocalDateTime {
-        LocalDateTime {
-            date: date,
-            time: time,
+    pub fn new(date: LocalDate, time: LocalTime) -> Self {
+        Self {
+            date,
+            time,
         }
     }
 
@@ -838,9 +838,9 @@ impl LocalDateTime {
 
     /// Creates a new date-time stamp set to the current time.
     #[cfg_attr(target_os = "redox", allow(unused_unsafe))]
-    pub fn now() -> LocalDateTime {
+    pub fn now() -> Self {
         let (s, ms) = unsafe { sys_time() };
-        LocalDateTime::at_ms(s, ms)
+        Self::at_ms(s, ms)
     }
 
     pub fn to_instant(&self) -> Instant {
@@ -848,7 +848,7 @@ impl LocalDateTime {
         Instant::at_ms(seconds, self.time.millisecond)
     }
 
-    pub fn add_seconds(&self, seconds: i64) -> LocalDateTime {
+    pub fn add_seconds(&self, seconds: i64) -> Self {
         Self::from_instant(self.to_instant() + Duration::of(seconds))
     }
 }
@@ -875,30 +875,30 @@ impl fmt::Debug for LocalDateTime {
 }
 
 impl Add<Duration> for LocalDateTime {
-    type Output = LocalDateTime;
+    type Output = Self;
 
-    fn add(self, duration: Duration) -> LocalDateTime {
-        LocalDateTime::from_instant(self.to_instant() + duration)
+    fn add(self, duration: Duration) -> Self {
+        Self::from_instant(self.to_instant() + duration)
     }
 }
 
 impl Sub<Duration> for LocalDateTime {
-    type Output = LocalDateTime;
+    type Output = Self;
 
-    fn sub(self, duration: Duration) -> LocalDateTime {
-        LocalDateTime::from_instant(self.to_instant() - duration)
+    fn sub(self, duration: Duration) -> Self {
+        Self::from_instant(self.to_instant() - duration)
     }
 }
 
 
-/// A **YMD** is an implementation detail of LocalDate. It provides
-/// helper methods relating to the construction of LocalDate instances.
+/// A **YMD** is an implementation detail of `LocalDate`. It provides
+/// helper methods relating to the construction of `LocalDate` instances.
 ///
-/// The main difference is that while all LocalDates get checked for
-/// validity before they are used, there is no such check for YMD. The
-/// interface to LocalDate ensures that it should be impossible to
+/// The main difference is that while all `LocalDate` values get checked
+/// for validity before they are used, there is no such check for `YMD`.
+/// The interface to `LocalDate` ensures that it should be impossible to
 /// create an instance of the 74th of March, for example, but you’re
-/// free to create such an instance of YMD. For this reason, it is not
+/// free to create such an instance of `YMD`. For this reason, it is not
 /// exposed to implementors of this library.
 #[derive(PartialEq, PartialOrd, Eq, Ord, Clone, Debug, Copy)]
 struct YMD {
@@ -916,7 +916,7 @@ impl YMD {
     /// This method returns a Result instead of exposing is_valid to
     /// the user, because the leap year calculations are used in both
     /// functions, so it makes more sense to only do them once.
-    pub fn to_days_since_epoch(&self) -> Result<i64, Error> {
+    fn to_days_since_epoch(&self) -> Result<i64, Error> {
         let years = self.year - 2000;
         let (leap_days_elapsed, is_leap_year) = Year(self.year).leap_year_calculations();
 
@@ -957,7 +957,7 @@ impl YMD {
     /// Whether the current year is a leap year should already have been
     /// calculated at this point, so the value is passed in rather than
     /// calculating it afresh.
-    pub fn is_valid(&self, is_leap_year: bool) -> bool {
+    fn is_valid(&self, is_leap_year: bool) -> bool {
         self.day >= 1 && self.day <= self.month.days_in_month(is_leap_year)
     }
 }
@@ -1000,14 +1000,11 @@ pub enum Error {
 
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.description())
+        write!(f, "datetime field out of range")
     }
 }
 
 impl ErrorTrait for Error {
-    fn description(&self) -> &str {
-        "datetime field out of range"
-    }
 }
 
 
@@ -1025,12 +1022,13 @@ pub enum Month {
     October = 10, November = 11, December  = 12,
 }
 
+#[allow(clippy::match_same_arms)]
 impl Month {
 
     /// Returns the number of days in this month, depending on whether it’s
     /// a leap year or not.
-    pub fn days_in_month(&self, leap_year: bool) -> i8 {
-        match *self {
+    pub fn days_in_month(self, leap_year: bool) -> i8 {
+        match self {
             January   => 31, February  => if leap_year { 29 } else { 28 },
             March     => 31, April     => 30,
             May       => 31, June      => 30,
@@ -1042,8 +1040,8 @@ impl Month {
 
     /// Returns the number of days that have elapsed in a year *before* this
     /// month begins, with no leap year check.
-    fn days_before_start(&self) -> i16 {
-        match *self {
+    fn days_before_start(self) -> i16 {
+        match self {
             January =>   0, February =>  31, March     =>  59,
             April   =>  90, May      => 120, June      => 151,
             July    => 181, August   => 212, September => 243,
@@ -1051,8 +1049,8 @@ impl Month {
         }
     }
 
-    pub fn months_from_january(&self) -> usize {
-        match *self {
+    pub fn months_from_january(self) -> usize {
+        match self {
             January =>   0, February =>   1, March     =>  2,
             April   =>   3, May      =>   4, June      =>  5,
             July    =>   6, August   =>   7, September =>  8,
@@ -1068,7 +1066,7 @@ impl Month {
     /// assert_eq!(Month::from_one(5), Ok(Month::May));
     /// assert!(Month::from_one(0).is_err());
     /// ```
-    pub fn from_one(month: i8) -> Result<Month, Error> {
+    pub fn from_one(month: i8) -> Result<Self, Error> {
         Ok(match month {
              1 => January,   2 => February,   3 => March,
              4 => April,     5 => May,        6 => June,
@@ -1086,7 +1084,7 @@ impl Month {
     /// assert_eq!(Month::from_zero(5), Ok(Month::June));
     /// assert!(Month::from_zero(12).is_err());
     /// ```
-    pub fn from_zero(month: i8) -> Result<Month, Error> {
+    pub fn from_zero(month: i8) -> Result<Self, Error> {
         Ok(match month {
             0 => January,   1 => February,   2 => March,
             3 => April,     4 => May,        5 => June,
@@ -1099,27 +1097,27 @@ impl Month {
 
 
 /// A named day of the week.
-///
-/// Sunday is Day 0. This seems to be a North American thing? It’s pretty
-/// much an arbitrary choice, and as you can’t use the from_zero method,
-/// it won’t affect you at all. If you want to change it, the only thing
-/// that should be affected is LocalDate::days_to_weekday.
 #[derive(PartialEq, Eq, Debug, Clone, Copy)]
 pub enum Weekday {
     Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday,
 }
 
+// Sunday is Day 0. This seems to be a North American thing? It’s pretty
+// much an arbitrary choice, and as you can’t use the `from_zero` method,
+// it won’t affect you at all. If you want to change it, the only thing
+// that should be affected is `LocalDate::days_to_weekday`.
+//
 // I’m not going to give weekdays an Ord instance because there’s no
 // real standard as to whether Sunday should come before Monday, or the
 // other way around. Luckily, they don’t need one, as the field is
 // ignored when comparing LocalDates.
 
 impl Weekday {
-    fn days_from_monday_as_one(&self) -> i8 {
-        match *self {
-            Sunday => 7,   Monday => 1,
-            Tuesday => 2,  Wednesday => 3,
-            Thursday => 4, Friday => 5,
+    fn days_from_monday_as_one(self) -> i8 {
+        match self {
+            Sunday   => 7,  Monday    => 1,
+            Tuesday  => 2,  Wednesday => 3,
+            Thursday => 4,  Friday    => 5,
             Saturday => 6,
         }
     }
@@ -1132,7 +1130,7 @@ impl Weekday {
     /// assert_eq!(Weekday::from_zero(4), Ok(Weekday::Thursday));
     /// assert!(Weekday::from_zero(7).is_err());
     /// ```
-    pub fn from_zero(weekday: i8) -> Result<Weekday, Error> {
+    pub fn from_zero(weekday: i8) -> Result<Self, Error> {
         Ok(match weekday {
             0 => Sunday,     1 => Monday,    2 => Tuesday,
             3 => Wednesday,  4 => Thursday,  5 => Friday,
@@ -1140,7 +1138,7 @@ impl Weekday {
         })
     }
 
-    pub fn from_one(weekday: i8) -> Result<Weekday, Error> {
+    pub fn from_one(weekday: i8) -> Result<Self, Error> {
         Ok(match weekday {
             7 => Sunday,     1 => Monday,    2 => Tuesday,
             3 => Wednesday,  4 => Thursday,  5 => Friday,
@@ -1153,9 +1151,7 @@ impl Weekday {
 /// Misc tests that don’t seem to fit anywhere.
 #[cfg(test)]
 mod test {
-    pub use super::{LocalDateTime, LocalDate, LocalTime, Month, Weekday, Year};
-    pub use cal::DatePiece;
-    pub use std::str::FromStr;
+    pub(crate) use super::{LocalDateTime, LocalDate, LocalTime, Month};
 
 
     #[test]
