@@ -1,30 +1,25 @@
-#![crate_name = "datetime"]
-#![crate_type = "rlib"]
-#![crate_type = "dylib"]
-
 #![warn(missing_copy_implementations)]
-#![warn(missing_debug_implementations)]
 //#![warn(missing_docs)]
+#![warn(nonstandard_style)]
+#![warn(trivial_numeric_casts)]
+#![warn(unreachable_pub)]
+#![warn(unused)]
 
-#![warn(trivial_casts, trivial_numeric_casts)]
-#![warn(unused_qualifications)]
-#![warn(unused_results)]
+#[cfg(feature="format")] extern crate locale;
+#[cfg(feature="format")] extern crate pad;
+#[cfg(feature="parse")]  extern crate iso8601;
 
-extern crate locale;
-extern crate libc;
-extern crate num_traits;
-extern crate pad;
-extern crate iso8601;
+#[cfg(all(unix, not(target_os = "macos"), not(target_os = "ios"), not(target_os = "redox")))]
+extern crate libc;  // used in the system module
 
-#[cfg(windows)] extern crate kernel32;
 #[cfg(windows)] extern crate winapi;
 
 
 mod cal;
 pub use cal::{DatePiece, TimePiece};
 pub use cal::datetime::{LocalDate, LocalTime, LocalDateTime, Month, Weekday, Year, YearMonth};
-pub use cal::fmt::custom as fmt;
-pub use cal::fmt::ISO;  // TODO: replace this with just a 'fmt' import
+#[cfg(feature="format")] pub use cal::fmt::custom as fmt;
+pub use cal::fmt::iso::ISO;  // TODO: replace this with just a 'fmt' import
 pub use cal::offset::{Offset, OffsetDateTime};
 pub use cal::zone::{TimeZone, ZonedDateTime};
 pub use cal::zone as zone;
