@@ -3,7 +3,9 @@
 use std::ops::Range;
 
 
-pub trait RangeExt {
+// TODO: replace this with the `range_contains` feature when it’s OK to use
+
+pub(crate) trait RangeExt {
 
     /// Returns whether this value exists within the given range of values.
     fn is_within(&self, range: Range<Self>) -> bool where Self: Sized;
@@ -16,4 +18,4 @@ impl<T> RangeExt for T where T: PartialOrd<T> {
     fn is_within(&self, range: Range<Self>) -> bool {
         *self >= range.start && *self < range.end
     }
-}
\ No newline at end of file
+}
